#include <string.h>
#include "keyd.h"
#include "actions.h"


/***************** Execution methods ****************/

extern char *mixer_names[];
extern void doMixerAction( action_t );
extern void doCDAction( action_t );
extern void doConsoleAction( action_t );

void execAction(action_t action)
{
	if(action && action->fn) (*action->fn)(action);
}


void doNullAction( action_t action )
{
}

void doModeAction( action_t action )
{
	enterMode(((mode_action_t)action)->mode);
}

void doExecAction( action_t action )
{
	exec_action_t exec = (exec_action_t)action;
	spawn( exec->uid, exec->gid, exec->cmd );
}

void doMultiAction( action_t action )
{
	multi_action_t multi = (multi_action_t)action;
	execAction( multi->first );
	execAction( multi->next );
}

/************** Construction methods *****************/

#define ACTION( type, fn ) ( (type *)makeAction(fn, sizeof(type)) )
#define ACTIONSTR( type, fn, str ) \
	    ((type *)makeAction(fn, sizeof(type) + (str?strlen(str):0)))
action_t makeAction(action_proc_t proc, int size)
{
	action_t act;
	act = s_zmalloc(size);
	act->len = size;
	act->fn = proc;
	return act;	
}

action_t makeNullAction()
{
	return ACTION( struct action_struct, doNullAction );
}

action_t makeModeAction(char *newmode)
{
	mode_action_t mode = ACTION( struct mode_action, doModeAction );
	addModeRef(mode, newmode);
	return (action_t)mode;
}

action_t makeExecAction(int uid, int gid, char *cmd)
{
	exec_action_t exec = ACTIONSTR( struct exec_action, doExecAction, cmd );
	exec->uid = uid;
	exec->gid = gid;
	strcpy(exec->cmd, cmd);
	free(cmd);
	return (action_t)exec;
}

action_t makeMixerAction(int rec, char *name, int op, int l, int r)
{
	int mixnum=-1,i;
	mixer_action_t mix;
	
	if(strcasecmp(name, "all")) {
		for( i=0; i< NUM_MIXERS; i++ ){
			if( !strcasecmp(name, mixer_names[i]) ) {
				mixnum = i;
				break;
			}
		}
		if( mixnum == -1 ) {
			log(LOG_ERR,"Unknown mixer device");
			free(name);
			return NULL;
		}
	}
	if( op == MIX_MUTE && mixnum == -1 ) 
		mixnum = 0; // to mute everything, just mute master volume

	mix = ACTION( struct mixer_action, doMixerAction );
	mix->rec = rec;
	mix->left = l;
	mix->right = r;
	mix->op = op;
	mix->mixer = mixnum;
	free(name);
	return (action_t)mix;
}

action_t makeCDAction( char *device, int op, int from, int to )
{
	cd_action_t cd;
	
	cd = ACTIONSTR( struct cd_action, doCDAction, device );
	if(device){
		strcpy(cd->device, device);
		free(device);
	}
	cd->op = op;
	cd->from = from;
	cd->to = to;
	return (action_t)cd;
}

action_t makeConsoleAction( int op, int con, int mark )
{
	console_action_t cons = ACTION( struct console_action, doConsoleAction );
	cons->op = op;
	cons->con = con;
	cons->mark= mark;
	return (action_t)cons;
}

action_t makeMultiAction( action_t a1, action_t a2 )
{
	multi_action_t multi = ACTION( struct multi_action, doMultiAction );
	multi->first = a1;
	multi->next = a2;
	return (action_t)multi;
}
