#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/signal.h>
#include <sys/ioctl.h>
#include "keyd.h"

extern int yydebug;
int consolefd = 0;
int autobg = 1;

#define SIGKBD SIGUSR1

void sigterm(int sig)
{
	log(LOG_INFO, "Caught signal %d, terminating",sig);
	restoreKeyTable();
	exit(0);
}

void sighup(int sig)
{
	/* Reload configuration */
	log(LOG_INFO, "Rereading configuration");
	installConfig(loadConfig( CONFIG_FILE ));
	signal(SIGHUP, sighup);
}

void sigkbd(int sig)
{
	int val;

	ioctl( consolefd, KDGETHOTKEY, &val );
	if(val != -1)
		execKey(val&0xFF, val&0x8000);
	signal(SIGKBD, sigkbd);
}

void initSignals()
{
	signal(SIGTERM, sigterm);
	signal(SIGINT, sigterm);
	signal(SIGSEGV, sigterm);
	signal(SIGBUS, sigterm);
	signal(SIGHUP, sighup);
	signal(SIGKBD, sigkbd);
}

int main(int argc, char *argv[] )
{
	keydrc_t rc;
	
	log( LOG_INFO, "Starting up...");
	rc = loadConfig( CONFIG_FILE );
	if(!rc){
		log(LOG_ERR, "Failed to load configuration, aborting");
		exit(1);
	}

	if(autobg) background();
	openlogfile();
	initSignals(rc);
	if(	ioctl(consolefd, KDSIGACCEPT, SIGKBD ) ){
		log(LOG_ERR, "Ioctl failed: KDSIGACCEPT. Yowch, aborting! (%s)", strerror(errno));
		exit(1);
	}

	installConfig(rc);

	hang();

	/* unreachable */
	log(LOG_ERR, "Huh? Quitting unexpectedly. Please report this");
	return 1; 
}
