
#ifndef __keyd_header_H
#define __keyd_header_H 1

#include <stdlib.h>
#include <syslog.h>
#include <stdio.h>
#include <linux/kd.h>
#include <linux/keyboard.h>
#include <linux/soundcard.h>

#define CONFIG_FILE "/etc/keyd.conf"

#define FALSE 0
#define TRUE 1
#define MAX_KEYS 256
#define MAX_MODES 32

#define MOD_SHIFT (1<<KG_SHIFT)
#define MOD_SHIFTL (1<<KG_SHIFTL)
#define MOD_SHIFTR (1<<KG_SHIFTR)
#define MOD_CTRL  (1<<KG_CTRL)
#define MOD_CTRLL (1<<KG_CTRLL)
#define MOD_CTRLR (1<<KG_CTRLR)
#define MOD_ALT   (1<<KG_ALT)
#define MOD_ALTL  (1<<KG_ALT)
#define MOD_ALTR  (1<<KG_ALTGR)

#define MOD_DOWN  0x1000
#define MOD_UP    0x2000
#define UPDOWN_MASK 0x3000


extern int usesyslog;
extern int consolefd;

#define log( lvl, str, args... ) \
      { if(usesyslog) syslog(lvl,str , ## args); \
      else logtofile( lvl, str , ## args); }
#define s_free( ptr ) { if(ptr) free(ptr); ptr = NULL; }

void *s_malloc(size_t);
void *s_zmalloc(size_t);
void openlogfile();
void logtofile(int lvl, char *str, ...); 
int background();
int spawn(int uid, int gid, char *cmd);
void hang();

typedef struct {
	int key;
	int mod;
	int orig;
} *keyent_t;

keyent_t makeKeyent(int code);

typedef struct keymode *keymode_t;

/******************** Actions ************************/

typedef struct action_struct {
	int len;
	void (*fn)( struct action_struct * );
	char data[0];
} *action_t;

typedef void (action_proc)( action_t );
typedef void (*action_proc_t)( action_t );

typedef struct mode_action {
	int len;
	action_proc_t fn;
	keymode_t mode;
} *mode_action_t;

/*******************************************************/

// Configuation information
typedef struct keydrc_struct {
	int numkeys, nummodes;
	char *mixerdev;
	char *cdromdev;
	keymode_t rootmode;
	keymode_t initialmode;
	action_t startup;
	keyent_t keytab[MAX_KEYS];
}	*keydrc_t;
extern keydrc_t keydrc;


// Called from parser
int addKey( keyent_t, action_t );
keymode_t addMode( char *name);
void addModeRef( mode_action_t, char * );
void resolveRefs( );
void setStartupAction( action_t );
void setEntryAction( action_t );
void setExitAction( action_t );
void openModeBlock( keymode_t );
void closeModeBlock( );
void extendLastAction( action_t );


keydrc_t loadConfig(char *);
void installConfig( keydrc_t );
void loadKeyTable( );
void restoreKeyTable( );
void execKey();
void execAction(action_t);

void enterMode( keymode_t );
void printModes( keydrc_t);


// lex/yacc stuff
extern int yydebug;
void yyrestart(FILE *f);
int yylex();
int yyerror(char *s);
int yyparse();
#endif
