#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <linux/cdrom.h>
#include <linux/soundcard.h>
#include "keyd.h"
#include "actions.h"

/***** Mixer Action ******/
int mixer_fd = -2;
char *mixer_dev = "/dev/mixer";
char *mixer_names[] = SOUND_DEVICE_NAMES;

#define MIXER_MAX 100
#define MIXERLIM(x) ((x)<0?0:((x)>MIXER_MAX?MIXER_MAX:(x)))
#define MIXERVAL(l,r) (MIXERLIM(l) | (MIXERLIM(r)<<8))
#define MIXERL(x) ((x)&0xFF)
#define MIXERR(x) (((x)>>8)&0xFF)

int mixer_mute[NUM_MIXERS];

void doMixerAction( mixer_action_t mixer )
{
	int val, oldval;
	
	if(mixer_fd == -2){
		mixer_fd = open(mixer_dev, O_RDONLY );
	}
	if(mixer_fd == -1) return;
	
	ioctl( mixer_fd, MIXER_READ( mixer->mixer), &oldval);
	
	if(mixer->rec){
		ioctl( mixer_fd, SOUND_MIXER_READ_RECSRC, &val);
		if(mixer->rec == -1) val &= ~(1<<mixer->mixer);
		else val |= (1<<mixer->mixer);
		ioctl( mixer_fd, SOUND_MIXER_WRITE_RECSRC, &val);
	}
	
	switch(mixer->op){
		case MIX_RECONLY:
			break;
		case MIX_SET:
			val = MIXERVAL(mixer->left, mixer->right);
			ioctl( mixer_fd, MIXER_WRITE( mixer->mixer ), &val);
			break;
		case MIX_DELTA:
			val = MIXERVAL( MIXERL(oldval)+mixer->left, MIXERR(oldval)+mixer->right );
			ioctl( mixer_fd, MIXER_WRITE( mixer->mixer ), &val);
			break;
		case MIX_MUTE:
			if(oldval == 0 && mixer_mute[mixer->mixer]!=0) {
				val = mixer_mute[mixer->mixer];
				mixer_mute[mixer->mixer] = 0;
			}
			else {
				val = 0;
				mixer_mute[mixer->mixer] = oldval;
			}
			ioctl( mixer_fd, MIXER_WRITE( mixer->mixer ), &val);
			break;
	}

}

/***** CD Drive Action *****/
#define ISPLAYING(sub) (sub.cdsc_audiostatus == CDROM_AUDIO_PLAY || \
                        sub.cdsc_audiostatus == CDROM_AUDIO_PAUSED )

void doCDAction( cd_action_t cd )
{
	struct cdrom_subchnl sub;
	struct cdrom_tochdr toc;
	struct cdrom_ti trk = {0,0,0,0};
	char *dev;
	int fd, play = 0, status;
	
	if(cd->device[0] == 0) dev = keydrc->cdromdev;
	else dev = cd->device;

	fd = open( dev, O_RDONLY|O_NONBLOCK );
	if(fd == -1){
		log( LOG_WARNING, "Unable to open CD-ROM device %s (%s)",dev,strerror(errno));
		return;
	}
	
	ioctl( fd, CDROMREADTOCHDR, &toc );
	
	sub.cdsc_format = CDROM_MSF;
	if(ioctl( fd, CDROMSUBCHNL, &sub )) {
		log( LOG_WARNING, "Ioctl CDROMSUBCHNL failed on %s (%s)",dev,strerror(errno));
		close(fd);
		return;
	}
	
	status = ioctl( fd, CDROM_DRIVE_STATUS, CDSL_CURRENT );
	if( status == -1 ){
		log( LOG_WARNING, "Ioctl CDROM_DRIVE_STATUS failed on %s (%s)",dev,strerror(errno));
		status = CDS_DISC_OK;
	}
	
	switch(cd->op) {
		case CD_STOP:
			ioctl( fd, CDROMSTOP );
			break;
		case CD_PAUSE:
			if( sub.cdsc_audiostatus == CDROM_AUDIO_PLAY )
				ioctl( fd, CDROMPAUSE );
			else if( sub.cdsc_audiostatus == CDROM_AUDIO_PAUSED )
				ioctl( fd, CDROMRESUME );
			break;
		case CD_STOPEJECT:
			if( ISPLAYING(sub) ) {
				ioctl( fd, CDROMSTOP );
				break;
			}
		case CD_EJECT:
			if( status == CDS_TRAY_OPEN )
				ioctl( fd, CDROMCLOSETRAY );
			else
				ioctl( fd, CDROMEJECT );
			break;
		case CD_LOCK:
			ioctl( fd, CDROM_LOCKDOOR, 1 );
			break;
		case CD_UNLOCK:
			ioctl( fd, CDROM_LOCKDOOR, 0 );
			break;
		case CD_NEXT:
		case CD_PREV:
			play = 1;
			if( ISPLAYING(sub) ){
				if(cd->op == CD_NEXT){
					trk.cdti_trk0 = sub.cdsc_trk + 1;
					if( trk.cdti_trk0 > toc.cdth_trk1 ) trk.cdti_trk0 = toc.cdth_trk0;
				} else {
					trk.cdti_trk0 = sub.cdsc_trk - 1;
					if( trk.cdti_trk0 < toc.cdth_trk0 ) trk.cdti_trk0 = toc.cdth_trk1;
				}
				trk.cdti_trk1 = toc.cdth_trk1;
			} else {
				trk.cdti_trk0 = toc.cdth_trk0;
				trk.cdti_trk1 = toc.cdth_trk1;
			}
	}
	if(play){
		ioctl( fd, CDROMSTOP );
		ioctl( fd, CDROMPLAYTRKIND, &trk );
	}
	close(fd);
}
