#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <stdarg.h>
#include <sys/wait.h>
#include <values.h>
#include "keyd.h"

int usesyslog = 0;
char *logfname = NULL;
FILE *logfile = stderr;

void openlogfile()
{
	if(usesyslog)
		openlog("keyd",LOG_CONS,LOG_DAEMON);
	else if(logfname)
		logfile = fopen(logfname,"wo");
	else
		logfile = stderr;
}

void logtofile(int lvl, char *str, ...)
{
	va_list ap;

	va_start(ap, str);
	fprintf(logfile,"keyd: ");
	vfprintf(logfile, str, ap );
	fprintf(logfile,"\n");
	va_end(ap);
}

int background()
{
	int i;
	
	i = fork();
	if(i==-1){
	  log(LOG_ERR, "Unable to background process: fork failed");
	  return FALSE;
	}
	if(i==0) return TRUE;
	exit(0);
}

void hang()
{
	while(1) sleep(MAXINT);
}

int spawn(int uid, int gid, char *cmd)
{
	int i;

	i = fork();
	if(i==-1){
		log(LOG_ERR, "Unable to execute command: fork failed");
		return FALSE;
	}
	if(i!=0){
		waitpid(i,NULL,0);
		return TRUE;
	}
	else {
		i = fork();
		if(i!=0) exit(0); // If that fork fails... I don't really want to know
		
		if(logfile != stderr) fclose(logfile);
		if(usesyslog) closelog();

		if( uid != -1 && getuid() == 0 ) setuid(uid);
		if( gid != -1 ) setgid(gid);
		
		execl( "/bin/sh", "/bin/sh", "-c", cmd, NULL );
		// SHOULD NOT GET HERE. NB: Can't really log this too easily :)
		exit(1);
	}
}

void *s_malloc(size_t len)
{
	void *ptr;
	
	ptr = malloc(len);
	if(ptr == NULL){
		log(LOG_ERR,"Memory allocation failure!");
		exit(1);
	}
	return ptr;
}

void *s_zmalloc(size_t len)
{
	void *ptr;
	ptr = s_malloc(len);
	memset(ptr, 0, len);
	return ptr;
}
